'   Subject: Fire Demo
'    Author: Liquidex
'     Email: LordQBASIC@aol.com
'   Version: 1.0
'      Date: 1.31.2000
' Copyright: (C) 2000 Alex Varanese of The Alphx Coding Group

DECLARE SUB BurnFrame (DstSeg%, DstOff%)
DECLARE SUB PrintLogo ()
DECLARE SUB InitTextMode ()
DECLARE SUB GenInit ()
DECLARE SUB DrawPoint (H%, V%, Col%, DstSeg%, DstOff%)
DECLARE SUB LoadPal (Filename$, PalSeg%, PalOff%)
DECLARE SUB ShutDown ()
DECLARE SUB InitGenData ()
DECLARE SUB ClrKBBuffer ()
DECLARE SUB ErrExit (ErrStr$)
DECLARE SUB BufferWaitAnyKey ()
DECLARE SUB SetPal (PalSeg%, PalOff%)

ON ERROR GOTO ErrHandler

DEFINT A-Z
'$DYNAMIC

CONST ScrnHRes& = 320, ScrnVRes& = 200
CONST MaxScrnH = ScrnHRes - 1, MaxScrnV = ScrnVRes - 1
CONST HalfScrnH = ScrnHRes& / 2 - 1, HalfScrnV = ScrnVRes& / 2 - 1
CONST FrameBufferByteSize& = ScrnHRes& * ScrnVRes&
CONST FrameBufferWordSize& = FrameBufferByteSize& / 2
CONST BlurClipBorderHPxlSize = 8, BlurClipBorderVPxlSize = 1

CONST Div5Deg! = 5.1

CONST FireFlowerRadScalar = 80
CONST FireFlowerPedalNum = 4

TYPE Rect2DType
 H1                             AS INTEGER
 V1                             AS INTEGER
 H2                             AS INTEGER
 V2                             AS INTEGER
END TYPE

DIM SHARED VOffMap(MaxScrnV) AS LONG
DIM SHARED FrameBuffer(FrameBufferWordSize + 1)
DIM SHARED ClipRgn AS Rect2DType
DIM SHARED DefPal(383)

DIM SHARED SinMap(359) AS SINGLE
DIM SHARED CosMap(359) AS SINGLE

DIM SHARED Div5Map(1275)

DIM SHARED FPS

GenInit

SecTimer! = TIMER + 1
FPS = 0
CurFireFlowerRotAng! = 0

DO
 FOR CurAng = 0 TO 359
  CurRadScalar = SIN((CurAng * FireFlowerPedalNum) + CurFireFlowerRotAng!) * FireFlowerRadScalar
  H = HalfScrnH + CosMap(CurAng) * CurRadScalar
  V = HalfScrnV - SinMap(CurAng) * CurRadScalar
 
  IF CINT(RND) = 1 THEN
   DrawPoint H, V, 255, VARSEG(FrameBuffer(0)), VARPTR(FrameBuffer(2))
  END IF
 NEXT CurAng

 IF CurFireFlowerRotAng! < 359 THEN
  CurFireFlowerRotAng! = CurFireFlowerRotAng! + .1
 ELSE
  CurFireFlowerRotAng! = 0
 END IF

 BurnFrame VARSEG(FrameBuffer(0)), VARPTR(FrameBuffer(2))

 PUT (0, 0), FrameBuffer, PSET

 ResetFrameCount = ResetFrameCount - 1
 IF TIMER <= SecTimer! THEN
  FPS = FPS + 1
 ELSEIF INKEY$ <> "" THEN EXIT DO
 END IF
LOOP

ShutDown

ErrHandler:
 InitTextMode
 SELECT CASE ERR
  CASE 6: ErrStr$ = "Overflow"
  CASE 7: ErrStr$ = "Out of memory"
  CASE 9: ErrStr$ = "Subscript out of range"
  CASE 11: ErrStr$ = "Division by zero"
  CASE 14: ErrStr$ = "Out of string space"
  CASE 28: ErrStr$ = "Out of stack space"
  CASE 51: ErrStr$ = "Internal error"
  CASE 52: ErrStr$ = "Bad file name/number"
  CASE 53: ErrStr$ = "File not found"
  CASE 54: ErrStr$ = "Bad file mode"
  CASE 62: ErrStr$ = "Data not found"
  CASE 76: ErrStr$ = "Path not found"
  CASE ELSE: ErrStr$ = "Unkown error"
 END SELECT
 ErrExit ErrStr$

REM $STATIC
SUB BufferWaitAnyKey
 ClrKBBuffer
 DO: LOOP UNTIL INKEY$ <> ""
END SUB

SUB BurnFrame (DstSeg, DstOff)
 DEF SEG = DstSeg
 CurPxlOff& = DstOff + 320
 FOR CurPxl& = 0 TO 63360
  Col = Div5Map(PEEK(CurPxlOff&) + PEEK(CurPxlOff& - 1) + PEEK(CurPxlOff& + 1) + PEEK(CurPxlOff& - 320) + PEEK(CurPxlOff& + 320)) - 1
  IF Col < 0 THEN Col = 0
  POKE (CurPxlOff& - 320), Col
  CurPxlOff& = CurPxlOff& + 1
 NEXT CurPxl&
 DEF SEG
END SUB

SUB ClrKBBuffer
 FOR I = 1 TO 10
  CurKeyASCII$ = INKEY$
 NEXT I
END SUB

SUB DrawPoint (H, V, Col, DstSeg, DstOff)
 IF H >= ClipRgn.H1 AND H <= ClipRgn.H2 AND V >= ClipRgn.V1 AND V <= ClipRgn.V2 THEN
  DEF SEG = DstSeg
  PxlAddr& = VOffMap(V) + H + DstOff
  Col = PEEK(PxlAddr&) + Col
  IF Col > 255 THEN Col = 255
  POKE (PxlAddr&), Col
  DEF SEG
 END IF
END SUB

SUB ErrExit (ErrStr$)
 PRINT
 PRINT "Error - " + ErrStr$ + "."
 PRINT "Press any key to exit..."
 BufferWaitAnyKey
 CLS
 END
END SUB

SUB GenInit
 InitTextMode
 CLS

 PrintLogo
 PRINT
 InitGenData
 PRINT
 PRINT "Press any key to begin..."
 BufferWaitAnyKey

 RANDOMIZE TIMER

 SCREEN 13
 LoadPal "Fire.PAL", VARSEG(DefPal(0)), VARPTR(DefPal(0))
 SetPal VARSEG(DefPal(0)), VARPTR(DefPal(0))
END SUB

SUB InitGenData
 PRINT "- Calculating vertical offset map... ";
 FOR V = 0 TO MaxScrnV
  VOffMap(V) = V * ScrnHRes
 NEXT V
 PRINT "done."

 PRINT "- Calculating trigometric function maps... ";
 FOR CurDeg = 0 TO 359
  CosMap(CurDeg) = COS(CurDeg)
  SinMap(CurDeg) = SIN(CurDeg)
 NEXT CurDeg
 PRINT "done."

 PRINT "- Calculating effect maps... ";
 FOR X = 0 TO 1275
  Div5Map(X) = X / Div5Deg!
 NEXT X
 PRINT "done."

 PRINT "- Initializing misc. engine data... ";
 RANDOMIZE TIMER
 FrameBuffer(0) = 2560
 FrameBuffer(1) = 200
 ClipRgn.H1 = BlurClipBorderHPxlSize
 ClipRgn.V1 = BlurClipBorderVPxlSize
 ClipRgn.H2 = MaxScrnH - BlurClipBorderHPxlSize
 ClipRgn.V2 = MaxScrnV - BlurClipBorderVPxlSize
 PRINT "done."
END SUB

SUB InitTextMode
 SCREEN 0
 WIDTH 80, 25
END SUB

SUB LoadPal (Filename$, PalSeg, PalOff)
 DEF SEG = PalSeg
 TempPalOff = PalOff
 BLOAD Filename$, TempPalOff
 DEF SEG
END SUB

SUB PrintLogo
 PRINT "Fire Demo v1.0"
 PRINT "Copyright (C) 2000 The Alphx Coding Group"
 PRINT "Written by Alex Varanese"
 PRINT STRING$(80, "")
END SUB

SUB SetPal (PalSeg, PalOff)
 DEF SEG = PalSeg
 CurPalOff = PalOff
 FOR CurCol = 0 TO 255
  OUT &H3C8, CurCol
  OUT &H3C9, PEEK(CurPalOff)
  OUT &H3C9, PEEK(CurPalOff + 1)
  OUT &H3C9, PEEK(CurPalOff + 2)
  CurPalOff = CurPalOff + 3
 NEXT CurCol
 DEF SEG
END SUB

SUB ShutDown
 InitTextMode
 PrintLogo
 PRINT
 PRINT "Thanks for trying out this demo!"
 PRINT
 PRINT "FPS:"; FPS
 END
END SUB

